
# THIS FILE IS GENERATED FROM OPENMM SETUP.PY
short_version = '7.7.0'
version = '7.7.0'
full_version = '7.7.0.dev-130124a'
git_revision = '130124a3f9277b054ec40927360a6ad20c8f5fa6'
release = False
openmm_library_path = r'D:/bld/openmm_1646185643413/_h_env/Library/lib'

if not release:
    version = full_version
